import datetime
import json
import logging
import concurrent.futures
from typing import Union

from .engine import Engine
from .host import Host
from .utils import generate_name

logging.basicConfig(level=logging.INFO)


class Scanner:
    """
    Scanner
    """
    def __init__(self, target: str, engines_paths: list = list('hi'), word_list_path: str = None,
                 output_file_path: str = None,
                 max_domains: int = None, max_levels: int = 5, engine_timeout: int = None, scan_ports: list = None,
                 workers: int = 1,
                 log_level: str = 'INFO'):
        self.target = Host(target)

        self.engines_paths = engines_paths

        self.max_domains = max_domains
        self.max_levels = max_levels
        self.scan_ports = scan_ports
        self.engine_timeout = engine_timeout
        self.engines = []
        self.domains = []
        self._target_set = set()

        self.log = logging.getLogger('dnz.Scanner')
        # self.log.setLevel(log_level)
        self.scanners = []

    def add_scanner(self, engine: Union[str, Engine]):
        """
        Add engine to list of engines to be run
        """
        if not isinstance(engine, Engine):
            raise TypeError('Engines must be instances of the Engine class')

        if engine not in self.engines:
            self.engines.append(engine)
            self.log.debug('Engine added to list of engines')
        else:
            self.log.debug('')

    def remove_engine(self, engine: Union[str, Engine]):
        """
        Remove engine from list of engines to be run
        """
        if not isinstance(engine, Engine):
            raise TypeError('Engines must be instances of the Engine class')

        if engine in self.engines:
            self.engines.remove(engine)
            self.log.debug('Engine removed from list of engines')
        else:
            self.log.debug('')

    def run(self, threaded: bool = False):
        if self.output_file_path:
            self.log.debug('Opening output path at %s', self.output_file_path)
            self.output_file_handler = open(self.output_file_path, 'w')

        # if self.workers > 0:
        #     with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
        #         future_to_url = {executor.submit(load_url, url, 60): url for url in URLS}
        #         for future in concurrent.futures.as_completed(future_to_url):
        #             url = future_to_url[future]
        #             try:
        #                 data = future.result()
        #             except Exception as exc:
        #                 print('%r generated an exception: %s' % (url, exc))
        #             else:
        #                 print('%r page is %d bytes' % (url, len(data)))

        for engine in self.engines:
            for host in engine.run():
                if isinstance(host, str):
                    valid_domain = True
                elif isinstance(host, list):
                    for item in host:
                        if isinstance(item, str):
                            valid_domain = True
                        else:
                            valid_domain = False
                else:
                    valid_domain = False

                if valid_domain:
                    self.target_set.add(host)
                    self.domains.append(host)
                    if self.output_file_path:
                        data = ''
                        json.dump(data, self.output_file_handler, indent=4)

        if self.scan_ports:
            for host in self.domains:
                host.scan_ports(self.scan_ports)

                if self.output_file_path:
                    data = ''
                    json.dump(data, self.output_file_handler, indent=4)

    def stop(self):
        if self.output_file_path:
            self.output_file_handler.close()
            self.log.debug('Close output file at %s', self.output_file_path)

    @property
    def targets(self):
        return list(self.target_set)

    def clear_targets(self):
        self.target_set = set()

    # def write_rows(self):
    #     fieldnames = ['first_name', 'last_name', 'Grade']
    #     writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
    #     writer.writeheader()
    #     writer.writerow({'Grade': 'B', 'first_name': 'Alex', 'last_name': 'Brian'})
    #     writer.writerows([{'Grade': 'B', 'first_name': 'Alex', 'last_name': 'Brian'},
    #                       {'Grade': 'A', 'first_name': 'Rachael',
    #                        'last_name': 'Rodriguez'},
    #                       {'Grade': 'C', 'first_name': 'Tom', 'last_name': 'smith'},
    #                       {'Grade': 'B', 'first_name': 'Jane', 'last_name': 'Oscar'},
    #                       {'Grade': 'A', 'first_name': 'Kennzy', 'last_name': 'Tim'}])

    def __str__(self):
        return self.target.fqdn

    def __repr__(self):
        return '<Scanner: {}>'.format(self.target.fqdn)


class Scan:
    def __init__(self):
        self.name = generate_name()
        self.start_date = datetime.datetime.now(datetime.timezone.utc)
        self.file_name = ''

    def __str__(self):
        return self.name

    def __repr__(self):
        return '<Scan: {}>'.format(self.name)



import logging
import abc

from itertools import chain


class Engine(abc.ABC):
    """
    Abstract class that all engines must inherit
    """
    name = None

    def __init__(self, delay: int = 5000, settings: dict = dict):
        self.delay = delay
        self.settings = settings
        self.log = logging.getLogger('dnz.engine.{}'.format(__class__))

    @abc.abstractmethod
    def run(self, host: str):
        pass

    def __str__(self):
        return self.name

    def __repr__(self):
        return '<Engine: {}>'.format(self.name)


def available_engines(cls: object = Engine):
    """
    List engines available for scanning
    """
    return list(
        chain.from_iterable(
            [list(chain.from_iterable([[x], available_engines(x)])) for x in cls.__subclasses__()]
        )
    )


def get_engine(name: str):
    engines = available_engines()
    for engine in engines:
        if engine.name == name:
            return engine



from dnz import Engine
from dnz.host import Host


class WordListEngine(Engine):
    name = 'word_list'

    def __init__(self, word_list_path):
        self.word_list_path = word_list_path
        self.word_list = []
        self.target_set = set()

        if self.word_list_path:
            with open(self.word_list_path, 'r'):
                self._open_word_list()

    def run(self, domain: str):
        self.generate_targets(domain)
        return list(self.target_set)

    def _open_word_list(self):
        with open(self.word_list_path, 'r') as fh:
            self.word_list = fh.read().splitlines()

    def reload_word_list(self):
        """
        Reload word list
        """
        self._open_word_list()

    def additional_sub_domains(self, domain: str, limit: int = None):
        for word in self.word_list[:limit]:
            new_target = Host(domain)
            new_target.add_sub_domain(word)
            self.target_set.add(new_target.fqdn)

    def numbered_sub_domains(self, domain: str, separator: str = '-', max_num: int = 10):
        if Host(domain).sub_domain:
            for num in range(1, max_num + 1):
                new_target = Host(domain)
                new_sub_domain = new_target.sub_domain + separator + str(num)
                new_target.replace_sub_domain(new_sub_domain)
                self.target_set.add(new_target.fqdn)

                new_target = Host(domain)
                new_sub_domain = new_target.sub_domain + str(num)
                new_target.replace_sub_domain(new_sub_domain)
                self.target_set.add(new_target.fqdn)

    def dashed_sub_domains(self, domain: str, separator: str = '-', limit: int = None):
        if Host(domain).sub_domain:
            for word in self.word_list[:limit]:
                new_target = Host(domain)
                new_sub_domain = new_target.sub_domain + separator + word
                new_target.replace_sub_domain(new_sub_domain)
                self.target_set.add(new_target.fqdn)

                new_target = Host(domain)
                new_sub_domain = word + separator + new_target.sub_domain
                new_target.replace_sub_domain(new_sub_domain)
                self.target_set.add(new_target.fqdn)

    def generate_targets(self, domain: str, target_types='all'):
        """
        Generate additional domain targets
        """
        if target_types == 'all':
            target_types = ['additional', 'numbered', 'dashed']
        else:
            target_types = [target_types]

        for method in target_types:
            if callable(getattr(self, f'{method}_sub_domains')):
                getattr(self, f'{method}_sub_domains')(domain)



import datetime
import logging
import socket
from typing import List

import tldextract
from .utils import is_valid_hostname, ping
import dns.resolver

log = logging.getLogger(__name__)


class Host:
    def __init__(self, url: str):
        """

        Args:
            url:
        """
        self.__parse_url(url)
        self.verified = False
        self.last_scan = ''
        self.ports_scanned = {}

    def __parse_url(self, url: str):
        self.parts = tldextract.extract(url)

    @property
    def is_valid(self):
        """
        Whether the host is a valid hostname

        Returns:
            bool
        """
        return is_valid_hostname(self.fqdn)

    @property
    def sub_domain(self):
        """
        Subdomain
        Returns:

        """
        return self.parts.subdomain

    @property
    def domain(self):
        """
        Domain
        Returns:

        """
        return self.parts.domain

    @property
    def suffix(self):
        """
        Suffix
        Returns:

        """
        return self.parts.suffix

    @property
    def fqdn(self):
        """
        fqdn
        Returns:

        """
        return self.parts.fqdn

    def add_sub_domain(self, sub_domain: str):
        current_sub_domain = self.sub_domain.split('.')
        current_sub_domain.insert(0, sub_domain)
        new_sub_domain = '.'.join([item for item in current_sub_domain if item])
        updated_url = '{}.{}.{}'.format(
            new_sub_domain, self.domain, self.suffix)
        self.__parse_url(updated_url)

    def replace_sub_domain(self, sub_domain: str):
        updated_url = '{}.{}.{}'.format(
            sub_domain, self.domain, self.suffix)
        self.__parse_url(updated_url)

    def to_dict(self):
        pass

    @property
    def source(self):
        return {
            'original': '',
            'all': []
        }

    @property
    def resolvers_processed(self):
        return []

    @property
    def resolvers_count(self):
        return []

    @property
    def resolvers_pct(self):
        return []

    @property
    def open_ports(self):
        """

        Returns:

        """
        return [port for port in self.ports_scanned.keys() if self.ports_scanned[port]['is_open']]

    @property
    def closed_ports(self):
        """

        Returns:

        """
        return [port for port in self.ports_scanned.keys() if not self.ports_scanned[port]['is_open']]

    def scan_ports(self, ports: List[int]):
        """
        Scan list of ports for TCP connection
        Args:
            ports:

        Returns:

        """
        log.debug('Performing port scan for host %s on %s ports', self.fqdn, len(ports))
        for port in ports:
            is_open = self.__check_port(port)
            self.ports_scanned[port] = {'is_open': is_open, 'checked_at': datetime.datetime.now(datetime.timezone.utc)}
        log.debug('Completed port scan for host %s on %s ports', self.fqdn, len(ports))
        return self.ports_scanned

    def __check_port(self, port: int, timeout: int = 2):
        log.debug('Checking open port for host %s on %s', self.fqdn, port)
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(timeout)
        try:
            result = s.connect_ex((self.fqdn, port))
            s.close()
        except Exception as error:
            log.debug('Checking open port for host %s on %s', self.fqdn, port)
            return False
        if result == 0:
            log.debug('Checking open port for host %s on %s', self.fqdn, port)
            return True
        else:
            log.debug('Checking open port for host %s on %s', self.fqdn, port)
            return False

    def ping(self):
        return ping(self.fqdn)

    def check_port(self, port: int):
        """
        Verify that host is up using TCP connection
        """
        return self.__check_port(port)

    def scan_results(self):
        return {'host': self.fqdn, 'engines': []}

    def dns(self):
        is_valid = False
        Resolver = dns.resolver.Resolver()
        Resolver.nameservers = ['8.8.8.8', '8.8.4.4']
        self.lock.acquire()
        return Resolver.query(self.fqdn, 'A')[0].to_text()

    def __str__(self):
        return self.fqdn

    def __repr__(self):
        return '<Host: {}>'.format(self.fqdn)



from dnz import Engine
from dnz.utils import is_valid_hostname
from typing import List
import dns.resolver

DEFAULT_RECORDS = [
    'NONE',
    'A',
    'ALIAS',
    'NS',
    'MD',
    'MF',
    'CNAME',
    'SOA',
    'MB',
    'MG',
    'MR',
    'NULL',
    'WKS',
    'PTR',
    'HINFO',
    'MINFO',
    'MX',
    'TXT',
    'RP',
    'AFSDB',
    'X25',
    'ISDN',
    'RT',
    'NSAP',
    'NSAP-PTR',
    'SIG',
    'KEY',
    'PX',
    'GPOS',
    'AAAA',
    'LOC',
    'NXT',
    'SRV',
    'NAPTR',
    'KX',
    'CERT',
    'A6',
    'DNAME',
    'OPT',
    'APL',
    'DS',
    'SSHFP',
    'IPSECKEY',
    'RRSIG',
    'NSEC',
    'DNSKEY',
    'DHCID',
    'NSEC3',
    'NSEC3PARAM',
    'TLSA',
    'HIP',
    'CDS',
    'CDNSKEY',
    'CSYNC',
    'SPF',
    'UNSPEC',
    'EUI48',
    'EUI64',
    'TKEY',
    'TSIG',
    'IXFR',
    'AXFR',
    'MAILB',
    'MAILA',
    'ANY',
    'URI',
    'CAA',
    'TA',
    'DLV',
]


class DNSResolverEngine(Engine):
    name = 'dns_resolver'

    def __init__(self, records: List[str] = None):
        if not records:
            self.records = DEFAULT_RECORDS
        else:
            self.records = records

    def run(self, domain: str):
        """
        Get all the records associated to domain parameter.
        """
        data = []
        for record in self.records:
            try:
                answers = dns.resolver.query(domain, record)
                for rdata in answers:
                    if is_valid_hostname(rdata.to_text()):
                        data.append(rdata.to_text())
            except Exception as e:
                pass

        return data



import re
import random
import subprocess
import platform
from typing import Any


def is_valid_hostname(hostname: str) -> bool:
    """
    Checks whether hostname is a valid hostname
    Adapted from https://stackoverflow.com/questions/2894902/check-for-a-valid-domain-name-in-a-string/2894922

    Args:
        hostname: Hostname (e.g. python.org)

    Returns:
        bool: True for valid, False otherwise.

    """
    valid_domain_re = r'^(?=.{1,253}$)(?!.*\.\..*)(?!\..*)([a-zA-Z0-9-]{,63}\.){,127}[a-zA-Z0-9-\.]{1,63}$'
    return re.match(valid_domain_re, hostname) is not None


def ping(hostname: str):
    """
    Perform network ping to host

    Args:
        hostname: Hostname (e.g. python.org)

    Returns:
        bool: True if ping is successful, false otherwise.

    """

    args = '-n 1' if platform.system().lower() == 'windows' else '-c 1'
    cmd = f'ping {args} {hostname}'
    need_sh = False if platform.system().lower() == 'windows' else True

    return subprocess.call(cmd, stdin=subprocess.PIPE, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT,
                           shell=need_sh) == 0


def generate_name(delimiter: str = '-', token_chars: str = '0123456789', token_len: int = 6):
    """
    Generate unique names based on nouns, adjectives, and number tokens
    e.g.: round-bread-144151

    Adapted from https://github.com/Atrox/haikunatorpy

    Args:
        delimiter: Delimiter to demarcate adjectives and nouns
        token_chars: Characters to use for unique token at end of name
        token_len: Length of unique token at end of name

    Returns:
        name
    """

    adjectives = [
        'aged', 'ancient', 'autumn', 'billowing', 'bitter', 'black', 'blue', 'bold',
        'broad', 'broken', 'calm', 'cold', 'cool', 'crimson', 'curly', 'damp',
        'dark', 'dawn', 'delicate', 'divine', 'dry', 'empty', 'falling', 'fancy',
        'flat', 'floral', 'fragrant', 'frosty', 'gentle', 'green', 'hidden', 'holy',
        'icy', 'jolly', 'late', 'lingering', 'little', 'lively', 'long', 'lucky',
        'misty', 'morning', 'muddy', 'mute', 'nameless', 'noisy', 'odd', 'old',
        'orange', 'patient', 'plain', 'polished', 'proud', 'purple', 'quiet', 'rapid',
        'raspy', 'red', 'restless', 'rough', 'round', 'royal', 'shiny', 'shrill',
        'shy', 'silent', 'small', 'snowy', 'soft', 'solitary', 'sparkling', 'spring',
        'square', 'steep', 'still', 'summer', 'super', 'sweet', 'throbbing', 'tight',
        'tiny', 'twilight', 'wandering', 'weathered', 'white', 'wild', 'winter', 'wispy',
        'withered', 'yellow', 'young'
    ]

    nouns = [
        'art', 'band', 'bar', 'base', 'bird', 'block', 'boat', 'bonus',
        'bread', 'breeze', 'brook', 'bush', 'butterfly', 'cake', 'cell', 'cherry',
        'cloud', 'credit', 'darkness', 'dawn', 'dew', 'disk', 'dream', 'dust',
        'feather', 'field', 'fire', 'firefly', 'flower', 'fog', 'forest', 'frog',
        'frost', 'glade', 'glitter', 'grass', 'hall', 'hat', 'haze', 'heart',
        'hill', 'king', 'lab', 'lake', 'leaf', 'limit', 'math', 'meadow',
        'mode', 'moon', 'morning', 'mountain', 'mouse', 'mud', 'night', 'paper',
        'pine', 'poetry', 'pond', 'queen', 'rain', 'recipe', 'resonance', 'rice',
        'river', 'salad', 'scene', 'sea', 'shadow', 'shape', 'silence', 'sky',
        'smoke', 'snow', 'snowflake', 'sound', 'star', 'sun', 'sun', 'sunset',
        'surf', 'term', 'thunder', 'tooth', 'tree', 'truth', 'union', 'unit',
        'violet', 'voice', 'water', 'waterfall', 'wave', 'wildflower', 'wind', 'wood'
    ]


    adjective = random.choice(adjectives)
    noun = random.choice(nouns)
    token = ''.join(random.choice(token_chars) for _ in range(token_len))

    sections = [adjective, noun, token]
    return delimiter.join(filter(None, sections))



import random
import uuid
from dnz import Engine


class DNSResolver(Engine):
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Language': 'en-US,en;q=0.8',
        'Accept-Encoding': 'gzip',
    }

    def __init__(self, record_type: str = 'A'):
        super(DNSResolver).__init__()

    # This process cannot block forever,  it  needs to check if its time to die.
    def add_nameserver(self, nameserver: str):
        """
        Add nameserver
        """
        keep_trying = True
        while not self.time_to_die and keep_trying:
            try:
                self.resolver_q.put(nameserver, timeout=1)
                trace("Added nameserver:", nameserver)
                keep_trying = False
            except Exception as e:
                if type(e) == Queue.Full or str(type(e)) == "<class 'queue.Full'>":
                    keep_trying = True

    def verify(self, nameserver_list: list):
        """
        Verify DNS resolvers work
        """
        added_resolver = False
        for server in nameserver_list:
            if self.time_to_die:
                break
            server = server.strip()
            if server:
                self.resolver.nameservers = [server]
                try:
                    # test_result = self.resolver.query(self.most_popular_website, "A")
                    # should throw an exception before this line.
                    if True:  # test_result:
                        # Only add the nameserver to the queue if we can detect wildcards.
                        if (self.find_wildcards(self.target)):  # and self.find_wildcards(".com")
                            # wildcards have been added to the set, it is now safe to be added to the queue.
                            # blocking queue,  this process will halt on put() when the queue is full:
                            self.add_nameserver(server)
                            added_resolver = True
                        else:
                            trace("Rejected nameserver - wildcard:", server)
                except Exception as e:
                    # Rejected server :(
                    trace("Rejected nameserver - unreliable:", server, type(e))
        return added_resolver

    # Only add the nameserver to the queue if we can detect wildcards.
    # Returns False on error.
    def find_wildcards(self, host: str):
        """
        Find wildcards on DNS resolver
        """
        # We want sovle the following three problems:
        # 1)The target might have a wildcard DNS record.
        # 2)The target maybe using geolocaiton-aware DNS.
        # 3)The DNS server we are testing may respond to non-exsistant 'A' records with advertizements.
        # I have seen a CloudFlare Enterprise customer with the first two conditions.
        try:
            # This is case #3,  these spam nameservers seem to be more trouble then they are worth.
            wildtest = self.resolver.query(uuid.uuid4().hex + ".com", "A")
            if len(wildtest):
                trace("Spam DNS detected:", host)
                return False
        except:
            pass
        test_counter = 8
        looking_for_wildcards = True
        while looking_for_wildcards and test_counter >= 0:
            looking_for_wildcards = False
            # Don't get lost, this nameserver could be playing tricks.
            test_counter -= 1
            try:
                testdomain = "%s.%s" % (uuid.uuid4().hex, host)
                wildtest = self.resolver.query(testdomain, self.record_type)
                # This 'A' record may contain a list of wildcards.
                if wildtest:
                    for w in wildtest:
                        w = str(w)
                        if w not in self.wildcards:
                            # wildcards were detected.
                            self.wildcards[w] = None
                            # We found atleast one wildcard, look for more.
                            looking_for_wildcards = True
            except Exception as e:
                if type(e) == dns.resolver.NXDOMAIN or type(e) == dns.name.EmptyLabel:
                    # not found
                    return True
                else:
                    # This resolver maybe flakey, we don't want it for our tests.
                    trace("wildcard exception:", self.resolver.nameservers, type(e))
                    return False
        # If we hit the end of our depth counter and,
        # there are still wildcards, then reject this nameserver because it smells bad.
        return (test_counter >= 0)

    def run(self, domain: str):
        """
        Run the resolver
        """
        # Every user will get a different set of resovlers, this helps redistribute traffic.
        random.shuffle(self.resolver_list)
        if not self.verify(self.resolver_list):
            # This should never happen,  inform the user.
            sys.stderr.write('Warning: No nameservers found, trying fallback list.\n')
            # Try and fix it for the user:
            self.verify(self.backup_resolver)
        # End of the resolvers list.
        try:
            self.resolver_q.put(False, timeout=1)
        except:
            pass

    def stop(self):
        """
        Stop the resolver
        """
        self.time_to_die = True

    def get_cname(q, target: str, resolved_out):
        global progress
        global lock
        global starttime
        global found
        global resolverName
        lock.acquire()
        progress += 1
        lock.release()
        if progress % 500 == 0:
            lock.acquire()
            left = linecount - progress
            secondspassed = (int(time.time()) - starttime) + 1
            amountpersecond = progress / secondspassed
            lock.release()
            timeleft = str(datetime.timedelta(seconds=int(left / amountpersecond)))
            print(
                colored("[*] {0}/{1} completed, approx {2} left".format(progress, linecount, timeleft),
                        "blue"))
        final_hostname = target
        result = list()
        result.append(target)
        resolver = dns.resolver.Resolver()
        if (resolverName is not None):  # if a DNS server has been manually specified
            resolver.nameservers = [resolverName]
        try:
            for rdata in resolver.query(final_hostname, 'CNAME'):
                result.append(rdata.target)
        except:
            pass
        if len(result) is 1:
            try:
                A = resolver.query(final_hostname, "A")
                if len(A) > 0:
                    result = list()
                    result.append(final_hostname)
                    result.append(str(A[0]))
            except:
                pass
        if len(result) > 1:  # will always have 1 item (target)
            if str(result[1]) in found:
                if found[str(result[1])] > 3:
                    return
                else:
                    found[str(result[1])] = found[str(result[1])] + 1
            else:
                found[str(result[1])] = 1
            resolved_out.write(str(result[0]) + ":" + str(result[1]) + "\n")
            resolved_out.flush()
            ext = tldextract.extract(str(result[1]))
            if ext.domain == "amazonaws":
                try:
                    for rdata in resolver.query(result[1], 'CNAME'):
                        result.append(rdata.target)
                except:
                    pass
            print(
                colored(
                    result[0],
                    "red") +
                " : " +
                colored(
                    result[1],
                    "green"))
            if len(result) > 2 and result[2]:
                print(
                    colored(
                        result[0],
                        "red") +
                    " : " +
                    colored(
                        result[1],
                        "green") +
                    ": " +
                    colored(
                        result[2],
                        "blue"))
        q.put(result)



from .engine import Engine, available_engines, get_engine
from .engines.scanning_services import *
from .engines.simple_dns import *
from .engines.web_search import *
from .engines.word_list import *
from .scanner import Scanner



import requests
from dnz import Engine
import re


class DNSdumpsterEngine(Engine):
    name = 'dns_dumpster'
    base_url = 'https://dnsdumpster.com/'

    def __init__(self):
        self.session = requests.Session()

    def run(self, domain: str):
        resp = self.req('GET', self.base_url)
        token = self.get_csrftoken(resp)
        params = {'csrfmiddlewaretoken': token, 'targetip': domain}
        post_resp = self.req('POST', self.base_url, params)
        return self.extract_domains(post_resp, domain)

    def req(self, req_method, url, params=None):
        params = params or {}
        headers = {}
        headers['Referer'] = 'https://dnsdumpster.com'
        try:
            if req_method == 'GET':
                resp = self.session.get(url, headers=headers, timeout=10)
            else:
                resp = self.session.post(url, data=params, headers=headers, timeout=10)
        except Exception as e:
            print(e)
            resp = None
        return resp

    def get_csrftoken(self, resp):
        csrf_regex = re.compile("<input type='hidden' name='csrfmiddlewaretoken' value='(.*?)' />", re.S)
        token = csrf_regex.findall(resp.text)[0]
        return token.strip()

    def extract_domains(self, resp, domain):
        extracted_domains = []
        tbl_regex = re.compile('<a name="hostanchor"><\/a>Host Records.*?<table.*?>(.*?)</table>', re.S)
        link_regex = re.compile('<td class="col-md-4">(.*?)<br>', re.S)
        try:
            results_tbl = tbl_regex.findall(resp.text)[0]
        except IndexError:
            results_tbl = ''
        links_list = link_regex.findall(results_tbl)
        links = list(set(links_list))
        for link in links:
            subdomain = link.strip()
            if not subdomain.endswith(domain):
                continue
            if subdomain and subdomain not in extracted_domains and subdomain != domain:
                extracted_domains.append(subdomain.strip())
        return extracted_domains


class VirusTotalEngine(Engine):
    name = 'virus_total'
    base_url = 'https://www.virustotal.com/en/domain/{domain}/information/'

    def run(self, domain: str):
        url = self.base_url.format(domain=domain)
        resp = requests.get(url)
        if not resp:
            return
        return self.extract_domains(resp, domain)

    def extract_domains(self, resp, domain):
        extracted_domains = []
        link_regx = re.compile('<div class="enum.*?">.*?<a target="_blank" href=".*?">(.*?)</a>', re.S)
        try:
            links = link_regx.findall(resp.text)
            for link in links:
                subdomain = link.strip()
                if not subdomain.endswith(domain):
                    continue
                if subdomain not in extracted_domains and subdomain != domain:
                    extracted_domains.append(subdomain.strip())
        except Exception:
            pass

        return extracted_domains


class ThreatCrowdEngine(Engine):
    name = 'threat_crowd'
    base_url = 'https://www.threatcrowd.org/searchApi/v2/domain/report/?domain={domain}'

    def run(self, domain: str):
        url = self.base_url.format(domain=domain)
        resp = requests.get(url)
        if not resp:
            return
        return self.extract_domains(resp, domain)

    def extract_domains(self, resp, domain):
        extracted_domains = []

        try:
            links = resp.json()['subdomains']
            for link in links:
                subdomain = link.strip()
                if not subdomain.endswith(domain):
                    continue
                if subdomain not in extracted_domains and subdomain != domain:
                    extracted_domains.append(subdomain.strip())
        except Exception as e:
            print(e)

        return extracted_domains


class CRTSearchEngine(Engine):
    name = 'crt_search'
    base_url = 'https://crt.sh/?q=%25.{domain}'

    def run(self, domain: str):
        url = self.base_url.format(domain=domain)
        resp = requests.get(url)
        if not resp:
            return
        return self.extract_domains(resp, domain)

    def extract_domains(self, resp, domain):
        extracted_domains = []

        link_regx = re.compile('<TD>(.*?)</TD>')
        try:
            links = link_regx.findall(resp.text)
            for link in links:
                subdomain = link.strip()
                if not subdomain.endswith(domain) or '*' in subdomain:
                    continue

                if '@' in subdomain:
                    subdomain = subdomain[subdomain.find('@') + 1:]

                if subdomain not in extracted_domains and subdomain != domain:
                    extracted_domains.append(subdomain.strip())
        except Exception as e:
            print(e)

        return extracted_domains


class PassiveDNSEngine(Engine):
    name = 'passive_dns'
    base_url = 'https://api.sublist3r.com/search.php?domain={domain}'

    def run(self, domain: str):
        url = self.base_url.format(domain=domain)
        resp = requests.get(url)
        if not resp:
            return
        return self.extract_domains(resp, domain)

    def extract_domains(self, resp, domain):
        extracted_domains = []
        try:
            subdomains = resp.json()
            for subdomain in subdomains:
                if subdomain not in extracted_domains and subdomain != domain:
                    extracted_domains.append(subdomain.strip())
        except Exception as e:
            pass

        return extracted_domains



import hashlib
import json
import multiprocessing
import random
# modules in standard library
import re
import sys
import threading
import time
from collections import Counter

from dnz import Engine

# external modules
import dns.resolver
import requests

# Python 2.x and 3.x compatiablity
if sys.version > '3':
    import urllib.parse as urlparse
    import urllib.parse as urllib
else:
    import urlparse
    import urllib

# In case you cannot install some of the required development packages
# there's also an option to disable the SSL warning:
# try:
#     import requests.packages.urllib3
#
#     requests.packages.urllib3.disable_warnings()
# except:
#     pass

# # Check if we are running this on windows platform
# is_windows = sys.platform.startswith('win')
#
#
# class enumratorBase(object):
#     def __init__(self, base_url, engine_name, domain, subdomains=None, silent=False, verbose=True):
#         subdomains = subdomains or []
#         self.domain = urlparse.urlparse(domain).netloc
#         self.session = requests.Session()
#         self.subdomains = []
#         self.timeout = 25
#         self.base_url = base_url
#         self.engine_name = engine_name
#         self.silent = silent
#         self.verbose = verbose
#         self.headers = {
#             'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36',
#             'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
#             'Accept-Language': 'en-US,en;q=0.8',
#             'Accept-Encoding': 'gzip',
#         }
#         self.print_banner()
#
#     def send_req(self, query, page_no=1):
#
#         url = self.base_url.format(query=query, page_no=page_no)
#         try:
#             resp = self.session.get(url, headers=self.headers, timeout=self.timeout)
#         except Exception:
#             resp = None
#         return self.get_response(resp)
#
#     def get_response(self, response):
#         if response is None:
#             return 0
#         return response.text if hasattr(response, "text") else response.content
#
#     def check_max_subdomains(self, count):
#         if self.MAX_DOMAINS == 0:
#             return False
#         return count >= self.MAX_DOMAINS
#
#     def check_max_pages(self, num):
#         if self.MAX_PAGES == 0:
#             return False
#         return num >= self.MAX_PAGES
#
#     # override
#     def extract_domains(self, resp):
#         """ chlid class should override this function """
#         return
#
#     # override
#     def check_response_errors(self, resp):
#         """ chlid class should override this function
#         The function should return True if there are no errors and False otherwise
#         """
#         return True
#
#     def should_sleep(self):
#         """Some enumrators require sleeping to avoid bot detections like Google enumerator"""
#         return
#
#     def generate_query(self):
#         """ chlid class should override this function """
#         return
#
#     def get_page(self, num):
#         """ chlid class that user different pagnation counter should override this function """
#         return num + 10
#
#     def enumerate(self, altquery=False):
#         flag = True
#         page_no = 0
#         prev_links = []
#         retries = 0
#
#         while flag:
#             query = self.generate_query()
#             count = query.count(self.domain)  # finding the number of subdomains found so far
#
#             # if they we reached the maximum number of subdomains in search query
#             # then we should go over the pages
#             if self.check_max_subdomains(count):
#                 page_no = self.get_page(page_no)
#
#             if self.check_max_pages(page_no):  # maximum pages for Google to avoid getting blocked
#                 return self.subdomains
#             resp = self.send_req(query, page_no)
#
#             # check if there is any error occured
#             if not self.check_response_errors(resp):
#                 return self.subdomains
#             links = self.extract_domains(resp)
#
#             # if the previous page hyperlinks was the similar to the current one, then maybe we have reached the last page
#             if links == prev_links:
#                 retries += 1
#                 page_no = self.get_page(page_no)
#
#                 # make another retry maybe it isn't the last page
#                 if retries >= 3:
#                     return self.subdomains
#
#             prev_links = links
#             self.should_sleep()
#
#         return self.subdomains
#
#
# class enumratorBaseThreaded(multiprocessing.Process, enumratorBase):
#     def __init__(self, base_url, engine_name, domain, subdomains=None, q=None, lock=threading.Lock(), silent=False,
#                  verbose=True):
#         subdomains = subdomains or []
#         enumratorBase.__init__(self, base_url, engine_name, domain, subdomains, silent=silent, verbose=verbose)
#         multiprocessing.Process.__init__(self)
#         self.lock = lock
#         self.q = q
#         return
#
#     def run(self):
#         domain_list = self.enumerate()
#         for domain in domain_list:
#             self.q.append(domain)
#
#
# # host,engine,date,ping,open_ports
#
# class GoogleSearchEngine(Engine):
#     name = 'google_search'
#     base_url = 'https://google.com/search?q={query}&btnG=Search&hl=en-US&biw=&bih=&gbv=1&start={page_no}&filter=0'
#     max_pages = 200
#
#     def extract_domains(self, resp):
#         links_list = list()
#         link_regx = re.compile('<cite.*?>(.*?)<\/cite>')
#         try:
#             links_list = link_regx.findall(resp)
#             for link in links_list:
#                 link = re.sub('<span.*>', '', link)
#                 if not link.startswith('http'):
#                     link = "http://" + link
#                 subdomain = urlparse.urlparse(link).netloc
#                 if subdomain and subdomain not in self.subdomains and subdomain != self.domain:
#                     if self.verbose:
#                         self.print_("%s%s: %s%s" % (R, self.engine_name, W, subdomain))
#                     self.subdomains.append(subdomain.strip())
#         except Exception:
#             pass
#         return links_list
#
#     def check_response_errors(self, resp):
#         if (type(resp) is str or type(resp) is unicode) and 'Our systems have detected unusual traffic' in resp:
#             self.print_(R + "[!] Error: Google probably now is blocking our requests" + W)
#             self.print_(R + "[~] Finished now the Google Enumeration ..." + W)
#             return False
#         return True
#
#     def generate_query(self):
#         if self.subdomains:
#             fmt = 'site:{domain} -www.{domain} -{found}'
#             found = ' -'.join(self.subdomains[:self.MAX_DOMAINS - 2])
#             query = fmt.format(domain=self.domain, found=found)
#         else:
#             query = "site:{domain} -www.{domain}".format(domain=self.domain)
#         return query
#
#
# class YahooSearchEngine(Engine):
#     name = 'yahoo_search'
#     base_url = 'https://search.yahoo.com/search?p={query}&b={page_no}'
#
#     def __init__(self, domain, subdomains=None, q=None, silent=False, verbose=True):
#         subdomains = subdomains or []
#
#         self.engine_name = "Yahoo"
#         self.MAX_DOMAINS = 10
#         self.MAX_PAGES = 0
#
#     def run(self):
#         pass
#
#     def extract_domains(self, resp):
#         link_regx2 = re.compile('<span class=" fz-.*? fw-m fc-12th wr-bw.*?">(.*?)</span>')
#         link_regx = re.compile('<span class="txt"><span class=" cite fw-xl fz-15px">(.*?)</span>')
#         links_list = []
#         try:
#             links = link_regx.findall(resp)
#             links2 = link_regx2.findall(resp)
#             links_list = links + links2
#             for link in links_list:
#                 link = re.sub("<(\/)?b>", "", link)
#                 if not link.startswith('http'):
#                     link = "http://" + link
#                 subdomain = urlparse.urlparse(link).netloc
#                 if not subdomain.endswith(self.domain):
#                     continue
#                 if subdomain and subdomain not in self.subdomains and subdomain != self.domain:
#                     if self.verbose:
#                         self.print_("%s%s: %s%s" % (R, self.engine_name, W, subdomain))
#                     self.subdomains.append(subdomain.strip())
#         except Exception:
#             pass
#
#         return links_list
#
#     def should_sleep(self):
#         return
#
#     def get_page(self, num):
#         return num + 10
#
#     def generate_query(self):
#         if self.subdomains:
#             fmt = 'site:{domain} -domain:www.{domain} -domain:{found}'
#             found = ' -domain:'.join(self.subdomains[:77])
#             query = fmt.format(domain=self.domain, found=found)
#         else:
#             query = "site:{domain}".format(domain=self.domain)
#         return query
#
#
# class AskSearchEngine(Engine):
#     name = 'ask_search'
#     base_url = 'http://www.ask.com/web?q={query}&page={page_no}&qid=8D6EE6BF52E0C04527E51F64F22C4534&o=0&l=dir&qsrc=998&qo=pagination'
#
#     def __init__(self, domain, subdomains=None, q=None):
#         subdomains = subdomains or []
#         self.MAX_DOMAINS = 11
#         self.MAX_PAGES = 0
#         enumratorBaseThreaded.__init__(self, base_url, self.engine_name, domain, subdomains, q=q, silent=silent,
#                                        verbose=verbose)
#         self.q = q
#         return
#
#     def run(self, domain: str):
#         pass
#
#     def extract_domains(self, resp):
#         links_list = list()
#         link_regx = re.compile('<p class="web-result-url">(.*?)</p>')
#         try:
#             links_list = link_regx.findall(resp)
#             for link in links_list:
#                 if not link.startswith('http'):
#                     link = "http://" + link
#                 subdomain = urlparse.urlparse(link).netloc
#                 if subdomain not in self.subdomains and subdomain != self.domain:
#                     if self.verbose:
#                         self.print_("%s%s: %s%s" % (R, self.engine_name, W, subdomain))
#                     self.subdomains.append(subdomain.strip())
#         except Exception:
#             pass
#
#         return links_list
#
#     def get_page(self, num):
#         return num + 1
#
#     def generate_query(self):
#         if self.subdomains:
#             fmt = 'site:{domain} -www.{domain} -{found}'
#             found = ' -'.join(self.subdomains[:self.MAX_DOMAINS])
#             query = fmt.format(domain=self.domain, found=found)
#         else:
#             query = "site:{domain} -www.{domain}".format(domain=self.domain)
#
#         return query
#
#
# class BingSearchEngine(Engine):
#     name = 'bing_search'
#     base_url = 'https://www.bing.com/search?q={query}&go=Submit&first={page_no}'
#
#     def __init__(self, domain, subdomains=None, q=None, silent=False, verbose=True):
#         subdomains = subdomains or []
#
#         self.engine_name = "Bing"
#         self.MAX_DOMAINS = 30
#         self.MAX_PAGES = 0
#         enumratorBaseThreaded.__init__(self, base_url, self.engine_name, domain, subdomains, q=q, silent=silent)
#         self.q = q
#         self.verbose = verbose
#         return
#
#     def run(self, domain: str):
#         pass
#
#     def extract_domains(self, resp):
#         links_list = list()
#         link_regx = re.compile('<li class="b_algo"><h2><a href="(.*?)"')
#         link_regx2 = re.compile('<div class="b_title"><h2><a href="(.*?)"')
#         try:
#             links = link_regx.findall(resp)
#             links2 = link_regx2.findall(resp)
#             links_list = links + links2
#
#             for link in links_list:
#                 link = re.sub('<(\/)?strong>|<span.*?>|<|>', '', link)
#                 if not link.startswith('http'):
#                     link = "http://" + link
#                 subdomain = urlparse.urlparse(link).netloc
#                 if subdomain not in self.subdomains and subdomain != self.domain:
#                     if self.verbose:
#                         self.print_("%s%s: %s%s" % (R, self.engine_name, W, subdomain))
#                     self.subdomains.append(subdomain.strip())
#         except Exception:
#             pass
#
#         return links_list
#
#     def generate_query(self):
#         if self.subdomains:
#             fmt = 'domain:{domain} -www.{domain} -{found}'
#             found = ' -'.join(self.subdomains[:self.MAX_DOMAINS])
#             query = fmt.format(domain=self.domain, found=found)
#         else:
#             query = "domain:{domain} -www.{domain}".format(domain=self.domain)
#         return query
#
#
# class BaiduSearchEngine(Engine):
#     name = 'baidu_search'
#     base_url = 'https://www.baidu.com/s?pn={page_no}&wd={query}&oq={query}'
#     MAX_PAGES = 760
#
#     def __init__(self, domain, subdomains=None, q=None, silent=False, verbose=True):
#         subdomains = subdomains or []
#
#         self.engine_name = "Baidu"
#         self.MAX_DOMAINS = 2
#         self.
#         enumratorBaseThreaded.__init__(self, base_url, self.engine_name, domain, subdomains, q=q, silent=silent,
#                                        verbose=verbose)
#         self.querydomain = self.domain
#         self.q = q
#         return
#
#     def run(self, domain: str):
#         pass
#
#     def extract_domains(self, resp):
#         links = list()
#         found_newdomain = False
#         subdomain_list = []
#         link_regx = re.compile('<a.*?class="c-showurl".*?>(.*?)</a>')
#         try:
#             links = link_regx.findall(resp)
#             for link in links:
#                 link = re.sub('<.*?>|>|<|&nbsp;', '', link)
#                 if not link.startswith('http'):
#                     link = "http://" + link
#                 subdomain = urlparse.urlparse(link).netloc
#                 if subdomain.endswith(self.domain):
#                     subdomain_list.append(subdomain)
#                     if subdomain not in self.subdomains and subdomain != self.domain:
#                         found_newdomain = True
#                         if self.verbose:
#                             self.print_("%s%s: %s%s" % (R, self.engine_name, W, subdomain))
#                         self.subdomains.append(subdomain.strip())
#         except Exception:
#             pass
#         if not found_newdomain and subdomain_list:
#             self.querydomain = self.findsubs(subdomain_list)
#         return links
#
#     def findsubs(self, subdomains):
#         count = Counter(subdomains)
#         subdomain1 = max(count, key=count.get)
#         count.pop(subdomain1, "None")
#         subdomain2 = max(count, key=count.get) if count else ''
#         return (subdomain1, subdomain2)
#
#     def check_response_errors(self, resp):
#         return True
#
#     def should_sleep(self):
#         time.sleep(random.randint(2, 5))
#         return
#
#     def generate_query(self):
#         if self.subdomains and self.querydomain != self.domain:
#             found = ' -site:'.join(self.querydomain)
#             query = "site:{domain} -site:www.{domain} -site:{found} ".format(domain=self.domain, found=found)
#         else:
#             query = "site:{domain} -site:www.{domain}".format(domain=self.domain)
#         return query



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup_args = dict(
    name='dnz',
    version='0.1.0',
    author='Marc Ford',
    url='https://github.com/mfdeux/dnz',
    description='Client for determining domains/subdomains.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages(),
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'click',
        'tldextract',
        'requests',
        'dnspython'
    ],
    entry_points={
        'console_scripts': ['dnz=dnz.cli:cli'],
    }
)

setup(**setup_args)



import click
import dnz
import json
import sys


@click.group(context_settings=dict(
    ignore_unknown_options=True,
    allow_extra_args=True,
))
@click.pass_context
def cli(ctx, **kwargs):
    pass


@cli.command(context_settings=dict(
    ignore_unknown_options=True,
    allow_extra_args=True,
))
@click.option('--host', '-h', default='www.espn.com', multiple=True, help='Host')
@click.option('--out', '-o', help='Help file to write out to')
@click.option('--format', '-f', default='json', type=click.Choice(['json', 'csv', 'txt']), help='Format of writing to file')
@click.argument('engine')
@click.pass_context
def targets(ctx, **kwargs):
    extra_args = {}
    if ctx.args:
        for item in ctx.args:
            extra_args.update({item.split('=')[0].replace('--', '').replace('-', ''): item.split('=')[1]})

    engine_cls = dnz.get_engine(kwargs.get('engine'))

    if not engine_cls:
        click.echo(click.style(f'{kwargs.get("engine")} is not a valid engine', 'red'))
        sys.exit(1)

    try:
        engine = engine_cls(**extra_args)
    except TypeError:
        click.echo(click.style('An option was entered that is not allowed', 'red'))
        sys.exit(1)

    total_output = {}

    for host in kwargs.get('host'):
        try:
            output = engine.run(host)
            total_output.update({host: output})
        except Exception:
            raise

    if kwargs.get('format') == 'json':
        click.echo(json.dumps(total_output))
    elif kwargs.get('format') == 'csv':
        pass
    else:
        for k, v in total_output.items():
            click.echo('\n'.join(v))


if __name__ == '__main__':
    cli()
